@@ -75,11 +75,11 @@ const Urgency = {
 const rewriteRules = {
     'XChat': [
         { pattern:     /^XChat: Private message from: (\S*) \(.*\)$/,
-          replacement: '&lt;$1&gt;' },
+          replacement: '<$1>' },
         { pattern:     /^XChat: New public message from: (\S*) \((.*)\)$/,
-          replacement: '$2 &lt;$1&gt;' },
+          replacement: '$2 <$1>' },
         { pattern:     /^XChat: Highlighted message from: (\S*) \((.*)\)$/,
-          replacement: '$2 &lt;$1&gt;' }
+          replacement: '$2 <$1>' }
     ]
 };
 
@@ -150,7 +150,9 @@ NotificationDaemon.prototype = {
                 let uri = GLib.filename_to_uri(icon, null);
                 return textureCache.load_uri_async(uri, size, size);
             } else
-                return textureCache.load_icon_name(icon, St.IconType.FULLCOLOR, size);
+                return new St.Icon({ icon_name: icon,
+                                     icon_type: St.IconType.FULLCOLOR,
+                                     icon_size: size });
         } else if (hints.icon_data) {
             let [width, height, rowStride, hasAlpha,
                  bitsPerSample, nChannels, data] = hints.icon_data;
@@ -167,7 +169,9 @@ NotificationDaemon.prototype = {
                     stockIcon = 'gtk-dialog-error';
                     break;
             }
-            return textureCache.load_icon_name(stockIcon, St.IconType.FULLCOLOR, size);
+            return new St.Icon({ icon_name: stockIcon,
+                                 icon_type: St.IconType.FULLCOLOR,
+                                 icon_size: size });
         }
     },
 
@@ -201,8 +205,6 @@ NotificationDaemon.prototype = {
             return id;
         }
 
-        summary = GLib.markup_escape_text(summary, -1);
-
         let rewrites = rewriteRules[appName];
         if (rewrites) {
             for (let i = 0; i < rewrites.length; i++) {
@@ -277,7 +279,9 @@ NotificationDaemon.prototype = {
         let iconActor = this._iconForNotificationData(icon, hints, source.ICON_SIZE);
 
         if (notification == null) {
-            notification = new MessageTray.Notification(source, summary, body, { icon: iconActor });
+            notification = new MessageTray.Notification(source, summary, body,
+                                                        { icon: iconActor,
+                                                          bannerMarkup: true });
             ndata.notification = notification;
             notification.connect('clicked', Lang.bind(this,
                 function(n) {
@@ -290,6 +294,7 @@ NotificationDaemon.prototype = {
             notification.connect('action-invoked', Lang.bind(this, this._actionInvoked, source, id));
         } else {
             notification.update(summary, body, { icon: iconActor,
+                                                 bannerMarkup: true,
                                                  clear: true });
         }
 
