@@ -1,7 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Clutter = imports.gi.Clutter;
-const Pango = imports.gi.Pango;
 const GConf = imports.gi.GConf;
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
@@ -48,10 +46,14 @@ PlaceInfo.prototype = {
         for (let i = 0; i < terms.length; i++) {
             let term = terms[i];
             let idx = this._lowerName.indexOf(term);
-            if (idx == 0)
-                return Search.MatchType.PREFIX;
-            else if (idx > 0)
-                mtype = Search.MatchType.SUBSTRING;
+            if (idx == 0) {
+                mtype = Search.MatchType.PREFIX;
+            } else if (idx > 0) {
+                if (mtype == Search.MatchType.NONE)
+                    mtype = Search.MatchType.SUBSTRING;
+            } else {
+                return Search.MatchType.NONE;
+            }
         }
         return mtype;
     },
@@ -77,7 +79,7 @@ PlaceDeviceInfo.prototype = {
 
     iconFactory: function(size) {
         let icon = this._mount.get_icon();
-        return St.TextureCache.get_default().load_gicon(icon, size);
+        return St.TextureCache.get_default().load_gicon(null, icon, size);
     },
 
     launch: function() {
@@ -107,7 +109,7 @@ PlaceDeviceInfo.prototype = {
                 this._mount.unmount_finish(res);
         } catch (e) {
             let message = _("Failed to unmount '%s'").format(o.get_name());
-            Main.overview.infoBar.setMessage(message,
+            Main.overview.shellInfo.setMessage(message,
                                              Lang.bind(this, this.remove),
                                              _("Retry"));
         }
@@ -135,7 +137,7 @@ PlacesManager.prototype = {
         let homeIcon = Shell.util_get_icon_for_uri (homeUri);
         this._home = new PlaceInfo('special:home', homeLabel,
             function(size) {
-                return St.TextureCache.get_default().load_gicon(homeIcon, size);
+                return St.TextureCache.get_default().load_gicon(null, homeIcon, size);
             },
             function() {
                 Gio.app_info_launch_default_for_uri(homeUri, global.create_app_launch_context());
@@ -148,7 +150,7 @@ PlacesManager.prototype = {
         let desktopIcon = Shell.util_get_icon_for_uri (desktopUri);
         this._desktopMenu = new PlaceInfo('special:desktop', desktopLabel,
             function(size) {
-                return St.TextureCache.get_default().load_gicon(desktopIcon, size);
+                return St.TextureCache.get_default().load_gicon(null, desktopIcon, size);
             },
             function() {
                 Gio.app_info_launch_default_for_uri(desktopUri, global.create_app_launch_context());
@@ -156,7 +158,9 @@ PlacesManager.prototype = {
 
         this._connect = new PlaceInfo('special:connect', _("Connect to..."),
             function (size) {
-                return St.TextureCache.get_default().load_icon_name('applications-internet', size);
+                return new St.Icon({ icon_name: 'applications-internet',
+                                     icon_type: St.IconType.FULLCOLOR,
+                                     icon_size: size });
             },
             function () {
                 new Shell.Process({ args: ['nautilus-connect-server'] }).run();
@@ -323,7 +327,7 @@ PlacesManager.prototype = {
 
             let item = new PlaceInfo('bookmark:' + bookmark, label,
                 function(size) {
-                    return St.TextureCache.get_default().load_gicon(icon, size);
+                    return St.TextureCache.get_default().load_gicon(null, icon, size);
                 },
                 function() {
                     Gio.app_info_launch_default_for_uri(bookmark, global.create_app_launch_context());
@@ -430,7 +434,9 @@ DashPlaceDisplayItem.prototype = {
         box.add(text, { expand: true, x_fill: true });
 
         if (info.isRemovable()) {
-            let removeIcon = St.TextureCache.get_default().load_icon_name ('media-eject', PLACES_ICON_SIZE);
+            let removeIcon = new St.Icon({ icon_name: 'media-eject',
+                                           icon_type: St.IconType.FULLCOLOR,
+                                           icon_size: PLACES_ICON_SIZE });
             let removeIconBox = new St.Clickable({ child: removeIcon,
                                                    reactive: true });
             box.add(removeIconBox);
@@ -573,8 +579,9 @@ PlaceSearchProvider.prototype = {
     },
 
     _searchPlaces: function(places, terms) {
-        let multipleResults = [];
+        let multiplePrefixResults = [];
         let prefixResults = [];
+        let multipleSubstringResults = [];
         let substringResults = [];
 
         terms = terms.map(String.toLowerCase);
@@ -582,17 +589,20 @@ PlaceSearchProvider.prototype = {
         for (let i = 0; i < places.length; i++) {
             let place = places[i];
             let mtype = place.matchTerms(terms);
-            if (mtype == Search.MatchType.MULTIPLE)
-                multipleResults.push(place.id);
+            if (mtype == Search.MatchType.MULTIPLE_PREFIX)
+                multiplePrefixResults.push(place.id);
             else if (mtype == Search.MatchType.PREFIX)
                 prefixResults.push(place.id);
+            else if (mtype == Search.MatchType.MULTIPLE_SUBSTRING)
+                multipleSubstringResults.push(place.id);
             else if (mtype == Search.MatchType.SUBSTRING)
                 substringResults.push(place.id);
         }
-        multipleResults.sort(this._compareResultMeta);
+        multiplePrefixResults.sort(this._compareResultMeta);
         prefixResults.sort(this._compareResultMeta);
+        multipleSubstringResults.sort(this._compareResultMeta);
         substringResults.sort(this._compareResultMeta);
-        return multipleResults.concat(prefixResults.concat(substringResults));
+        return multiplePrefixResults.concat(prefixResults.concat(multipleSubstringResults.concat(substringResults)));
     },
 
     getInitialResultSet: function(terms) {
