@@ -1,9 +1,6 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
-const GdkPixbuf = imports.gi.GdkPixbuf;
-const Gdk = imports.gi.Gdk;
-const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -23,9 +20,6 @@ const FOCUS_ANIMATION_TIME = 0.15;
 
 const WINDOW_DND_SIZE = 256;
 
-const FRAME_COLOR = new Clutter.Color();
-FRAME_COLOR.from_pixel(0xffffffff);
-
 const SCROLL_SCALE_AMOUNT = 100 / 5;
 
 const LIGHTBOX_FADE_TIME = 0.1;
@@ -57,11 +51,6 @@ function _clamp(value, min, max) {
     return Math.max(min, Math.min(max, value));
 }
 
-// Spacing between workspaces. At the moment, the same spacing is used
-// in both zoomed-in and zoomed-out views; this is slightly
-// metaphor-breaking, but the alternatives are also weird.
-const GRID_SPACING = 15;
-const FRAME_SIZE = GRID_SPACING / 3;
 
 function ScaledPoint(x, y, scaleX, scaleY) {
     [this.x, this.y, this.scaleX, this.scaleY] = arguments;
@@ -143,7 +132,10 @@ WindowClone.prototype = {
         if (this.inDrag || this._zooming)
             // We'll fix up the stack after the drag/zooming
             return;
-        this.actor.raise(this._stackAbove);
+        if (this._stackAbove == null)
+            this.actor.lower_bottom();
+        else
+            this.actor.raise(this._stackAbove);
     },
 
     destroy: function () {
@@ -250,11 +242,13 @@ WindowClone.prototype = {
         this.emit('zoom-end');
 
         this.actor.reparent(this._origParent);
+        if (this._stackAbove == null)
+            this.actor.lower_bottom();
         // If the workspace has been destroyed while we were reparented to
         // the stage, _stackAbove will be unparented and we can't raise our
         // actor above it - as we are bound to be destroyed anyway in that
         // case, we can skip that step
-        if (this._stackAbove && this._stackAbove.get_parent())
+        else if (this._stackAbove.get_parent())
             this.actor.raise(this._stackAbove);
 
         [this.actor.x, this.actor.y]             = this._zoomLocalOrig.getPosition();
@@ -286,82 +280,20 @@ WindowClone.prototype = {
         // We may not have a parent if DnD completed successfully, in
         // which case our clone will shortly be destroyed and replaced
         // with a new one on the target workspace.
-        if (this.actor.get_parent() != null)
-            this.actor.raise(this._stackAbove);
+        if (this.actor.get_parent() != null) {
+            if (this._stackAbove == null)
+                this.actor.lower_bottom();
+            else
+                this.actor.raise(this._stackAbove);
+        }
+
 
         this.emit('drag-end');
     }
 };
-
 Signals.addSignalMethods(WindowClone.prototype);
 
 
-function DesktopClone(window) {
-    this._init(window);
-}
-
-DesktopClone.prototype = {
-    _init : function(window) {
-        this.actor = new Clutter.Group({ reactive: true });
-
-        let background = new Clutter.Clone({ source: Main.background.source });
-        this.actor.add_actor(background);
-
-        if (window) {
-            this._desktop = new Clutter.Clone({ source: window.get_texture() });
-            this.actor.add_actor(this._desktop);
-            this._desktop.hide();
-        } else {
-            this._desktop = null;
-        }
-
-        this.actor.connect('button-release-event',
-                           Lang.bind(this, this._onButtonRelease));
-    },
-
-    zoomFromOverview: function(fadeInIcons) {
-        if (this._desktop == null)
-            return;
-
-        if (fadeInIcons) {
-            this._desktop.opacity = 0;
-            this._desktop.show();
-            Tweener.addTween(this._desktop,
-                             { opacity: 255,
-                               time: Overview.ANIMATION_TIME,
-                               transition: 'easeOutQuad' });
-        }
-    },
-
-    zoomToOverview: function(fadeOutIcons) {
-        if (this._desktop == null)
-            return;
-
-        if (fadeOutIcons) {
-            this._desktop.opacity = 255;
-            this._desktop.show();
-            Tweener.addTween(this._desktop,
-                             { opacity: 0,
-                               time: Overview.ANIMATION_TIME,
-                               transition: 'easeOutQuad',
-                               onComplete: Lang.bind(this,
-                                   function() {
-                                       this._desktop.hide();
-                                   })
-                             });
-        } else {
-            this._desktop.hide();
-        }
-    },
-
-    _onButtonRelease : function (actor, event) {
-        this.emit('selected', event.get_time());
-    }
-};
-
-Signals.addSignalMethods(DesktopClone.prototype);
-
-
 /**
  * @windowClone: Corresponding window clone
  * @parentActor: The actor which will be the parent of all overlay items
@@ -537,12 +469,10 @@ WindowOverlay.prototype = {
 
     _idleToggleCloseButton: function() {
         this._idleToggleCloseId = 0;
-        let [x, y, mask] = global.get_pointer();
-        let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE,
-                                                  x, y);
-        if (actor != this._windowClone.actor && actor != this.closeButton) {
+        if (!this._windowClone.actor.has_pointer &&
+            !this.closeButton.has_pointer)
             this.closeButton.hide();
-        }
+
         return false;
     },
 
@@ -556,23 +486,14 @@ WindowOverlay.prototype = {
 
     _onStyleChanged: function() {
         let titleNode = this.title.get_theme_node();
-
-        let [success, len] = titleNode.get_length('-shell-caption-spacing',
-                                                  false);
-        if (success)
-            this.title._spacing = len;
+        this.title._spacing = titleNode.get_length('-shell-caption-spacing');
 
         let closeNode = this.closeButton.get_theme_node();
-
-        [success, len] = closeNode.get_length('-shell-close-overlap',
-                                              false);
-        if (success)
-            this.closeButton._overlap = len;
+        this.closeButton._overlap = closeNode.get_length('-shell-close-overlap');
 
         this._parentActor.queue_relayout();
     }
 };
-
 Signals.addSignalMethods(WindowOverlay.prototype);
 
 const WindowPositionFlags = {
@@ -596,10 +517,20 @@ Workspace.prototype = {
         // Without this the drop area will be overlapped.
         this._windowOverlaysGroup.set_size(0, 0);
 
-        this.actor = new Clutter.Group();
+        this.actor = new Clutter.Group({ reactive: true });
         this.actor._delegate = this;
 
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
+        this.actor.connect('button-release-event', Lang.bind(this,
+            function(actor, event) {
+                // Only switch to the workspace when there's no application
+                // windows open. The problem is that it's too easy to miss
+                // an app window and get the wrong one focused.
+                if (this._windows.length == 0) {
+                    this.metaWorkspace.activate(event.get_time());
+                    Main.overview.hide();
+                }
+            }));
 
         // Items in _windowOverlaysGroup should not be scaled, so we don't
         // add them to this.actor, but to its parent whenever it changes
@@ -613,37 +544,12 @@ Workspace.prototype = {
         this.actor.height = global.screen_height;
         this.scale = 1.0;
 
-        let windows = global.get_windows().filter(this._isMyWindow, this);
-
-        // Find the desktop window
-        for (let i = 0; i < windows.length; i++) {
-            if (windows[i].get_window_type() == Meta.WindowType.DESKTOP) {
-                this._desktop = new DesktopClone(windows[i]);
-                break;
-            }
-        }
-        // If there wasn't one, fake it
-        if (!this._desktop)
-            this._desktop = new DesktopClone();
-
-        this._desktop.connect('selected',
-                              Lang.bind(this,
-                                        function(clone, time) {
-                                            // Only switch to the workspace when there's no application windows
-                                            // open (we always have one window for the desktop).  The problem
-                                            // is that it's too easy to miss an app window and get the wrong
-                                            // one focused.
-                                            if (this._windows.length == 1) {
-                                                this.metaWorkspace.activate(time);
-                                                Main.overview.hide();
-                                            }
-                                        }));
-        this.actor.add_actor(this._desktop.actor);
+        let windows = global.get_window_actors().filter(this._isMyWindow, this);
 
         // Create clones for remaining windows that should be
         // visible in the Overview
-        this._windows = [this._desktop];
-        this._windowOverlays = [ null ];
+        this._windows = [];
+        this._windowOverlays = [];
         for (let i = 0; i < windows.length; i++) {
             if (this._isOverviewWindow(windows[i])) {
                 this._addWindowClone(windows[i]);
@@ -662,8 +568,6 @@ Workspace.prototype = {
 
         this._visible = false;
 
-        this._frame = null;
-
         this.leavingOverview = false;
     },
 
@@ -725,9 +629,6 @@ Workspace.prototype = {
             this._lightbox.show();
         else
             this._lightbox.hide();
-
-        if (this._frame)
-            this._frame.set_opacity(showLightbox ? 150 : 255);
     },
 
     /**
@@ -748,32 +649,6 @@ Workspace.prototype = {
         this._lightbox.highlight(actor);
     },
 
-    // Mark the workspace selected/not-selected
-    setSelected : function(selected) {
-        // Don't draw a frame if we only have one workspace
-        if (selected && global.screen.n_workspaces > 1) {
-            if (this._frame)
-                return;
-
-            // FIXME: do something cooler-looking using clutter-cairo
-            this._frame = new Clutter.Rectangle({ color: FRAME_COLOR });
-            this.actor.add_actor(this._frame);
-            this._frame.set_position(this._desktop.actor.x - FRAME_SIZE / this.actor.scale_x,
-                                     this._desktop.actor.y - FRAME_SIZE / this.actor.scale_y);
-            this._frame.set_size(this._desktop.actor.width + 2 * FRAME_SIZE / this.actor.scale_x,
-                                 this._desktop.actor.height + 2 * FRAME_SIZE / this.actor.scale_y);
-            this._frame.lower_bottom();
-
-            this._framePosHandler = this.actor.connect('notify::scale-x', Lang.bind(this, this._updateFramePosition));
-        } else {
-            if (!this._frame)
-                return;
-            this.actor.disconnect(this._framePosHandler);
-            this._frame.destroy();
-            this._frame = null;
-        }
-    },
-
     /**
      * setReactive:
      * @reactive: %true iff the workspace should be reactive
@@ -781,14 +656,7 @@ Workspace.prototype = {
      * Set the workspace (desktop) reactive
      **/
     setReactive: function(reactive) {
-        this._desktop.actor.reactive = reactive;
-    },
-
-    _updateFramePosition : function() {
-        this._frame.set_position(this._desktop.actor.x - FRAME_SIZE / this.actor.scale_x,
-                                 this._desktop.actor.y - FRAME_SIZE / this.actor.scale_y);
-        this._frame.set_size(this._desktop.actor.width + 2 * FRAME_SIZE / this.actor.scale_x,
-                             this._desktop.actor.height + 2 * FRAME_SIZE / this.actor.scale_y);
+        this.actor.reactive = reactive;
     },
 
     _isCloneVisible: function(clone) {
@@ -799,7 +667,7 @@ Workspace.prototype = {
      * _getVisibleClones:
      *
      * Returns a list WindowClone objects where the clone isn't filtered
-     * out by any application filter.  The clone for the desktop is excluded.
+     * out by any application filter.
      * The returned array will always be newly allocated; it is not in any
      * defined order, and thus it's convenient to call .sort() with your
      * choice of sorting function.
@@ -807,7 +675,7 @@ Workspace.prototype = {
     _getVisibleClones: function() {
         let visible = [];
 
-        for (let i = 1; i < this._windows.length; i++) {
+        for (let i = 0; i < this._windows.length; i++) {
             let clone = this._windows[i];
 
             if (!this._isCloneVisible(clone))
@@ -819,7 +687,7 @@ Workspace.prototype = {
     },
 
     _resetCloneVisibility: function () {
-        for (let i = 1; i < this._windows.length; i++) {
+        for (let i = 0; i < this._windows.length; i++) {
             let clone = this._windows[i];
             let overlay = this._windowOverlays[i];
 
@@ -1018,9 +886,9 @@ Workspace.prototype = {
         let buttonOuterHeight, captionHeight;
         let buttonOuterWidth = 0;
 
-        if (this._windowOverlays[1]) {
-            [buttonOuterHeight, captionHeight] = this._windowOverlays[1].chromeHeights();
-            buttonOuterWidth = this._windowOverlays[1].chromeWidth() / this.scale;
+        if (this._windowOverlays[0]) {
+            [buttonOuterHeight, captionHeight] = this._windowOverlays[0].chromeHeights();
+            buttonOuterWidth = this._windowOverlays[0].chromeWidth() / this.scale;
         } else
             [buttonOuterHeight, captionHeight] = [0, 0];
         buttonOuterHeight /= this.scale;
@@ -1139,8 +1007,6 @@ Workspace.prototype = {
     },
 
     syncStacking: function(stackIndices) {
-        let desktopClone = this._windows[0];
-
         let visibleClones = this._getVisibleClones();
         visibleClones.sort(function (a, b) { return stackIndices[a.metaWindow.get_stable_sequence()] - stackIndices[b.metaWindow.get_stable_sequence()]; });
 
@@ -1148,7 +1014,7 @@ Workspace.prototype = {
             let clone = visibleClones[i];
             let metaWindow = clone.metaWindow;
             if (i == 0) {
-                clone.setStackAbove(desktopClone.actor);
+                clone.setStackAbove(null);
             } else {
                 let previousClone = visibleClones[i - 1];
                 clone.setStackAbove(previousClone.actor);
@@ -1168,8 +1034,8 @@ Workspace.prototype = {
         // be after the workspace animation finishes.
         let [cloneX, cloneY] = clone.actor.get_position();
         let [cloneWidth, cloneHeight] = clone.actor.get_size();
-        cloneX = this.gridX + this.scale * cloneX;
-        cloneY = this.gridY + this.scale * cloneY;
+        cloneX = this.x + this.scale * cloneX;
+        cloneY = this.y + this.scale * cloneY;
         cloneWidth = this.scale * clone.actor.scale_x * cloneWidth;
         cloneHeight = this.scale * clone.actor.scale_y * cloneHeight;
 
@@ -1183,7 +1049,7 @@ Workspace.prototype = {
     },
 
     _fadeInAllOverlays: function() {
-        for (let i = 1; i < this._windows.length; i++) {
+        for (let i = 0; i < this._windows.length; i++) {
             let clone = this._windows[i];
             let overlay = this._windowOverlays[i];
             if (this._showOnlyWindows != null && !(clone.metaWindow in this._showOnlyWindows))
@@ -1193,7 +1059,7 @@ Workspace.prototype = {
     },
 
     _hideAllOverlays: function() {
-        for (let i = 1; i< this._windows.length; i++) {
+        for (let i = 0; i < this._windows.length; i++) {
             let overlay = this._windowOverlays[i];
             overlay.hide();
         }
@@ -1208,8 +1074,8 @@ Workspace.prototype = {
         let wsHeight = this.actor.height * this.scale;
 
         let pointerHasMoved = (this._cursorX != x && this._cursorY != y);
-        let inWorkspace = (this.gridX < x && x < this.gridX + wsWidth &&
-                           this.gridY < y && y < this.gridY + wsHeight);
+        let inWorkspace = (this.x < x && x < this.x + wsWidth &&
+                           this.y < y && y < this.y + wsHeight);
 
         if (pointerHasMoved && inWorkspace) {
             // store current cursor position
@@ -1320,8 +1186,8 @@ Workspace.prototype = {
     },
 
     // check for maximized windows on the workspace
-    _haveMaximizedWindows: function() {
-        for (let i = 1; i < this._windows.length; i++) {
+    hasMaximizedWindows: function() {
+        for (let i = 0; i < this._windows.length; i++) {
             let metaWindow = this._windows[i].metaWindow;
             if (metaWindow.showing_on_its_workspace() &&
                 metaWindow.maximized_horizontally &&
@@ -1333,7 +1199,7 @@ Workspace.prototype = {
 
     // Animate the full-screen to Overview transition.
     zoomToOverview : function() {
-        this.actor.set_position(this.gridX, this.gridY);
+        this.actor.set_position(this.x, this.y);
         this.actor.set_scale(this.scale, this.scale);
 
         // Position and scale the windows.
@@ -1342,12 +1208,6 @@ Workspace.prototype = {
         else
             this.positionWindows(WindowPositionFlags.ZOOM);
 
-        let active = global.screen.get_active_workspace();
-        let fadeInIcons = (Main.overview.animationInProgress &&
-                           active == this.metaWorkspace &&
-                           !this._haveMaximizedWindows());
-        this._desktop.zoomToOverview(fadeInIcons);
-
         this._visible = true;
     },
 
@@ -1365,7 +1225,7 @@ Workspace.prototype = {
                                                                            this._doneLeavingOverview));
 
         // Position and scale the windows.
-        for (let i = 1; i < this._windows.length; i++) {
+        for (let i = 0; i < this._windows.length; i++) {
             let clone = this._windows[i];
 
             clone.zoomFromOverview();
@@ -1394,77 +1254,9 @@ Workspace.prototype = {
             }
         }
 
-        let active = global.screen.get_active_workspace();
-        let fadeOutIcons = (active == this.metaWorkspace &&
-                            !this._haveMaximizedWindows());
-        this._desktop.zoomFromOverview(fadeOutIcons);
-
         this._visible = false;
     },
 
-    // Animates grid shrinking/expanding when a row or column
-    // of workspaces is added or removed
-    resizeToGrid : function (oldScale) {
-        this._hideAllOverlays();
-        Tweener.addTween(this.actor,
-                         { x: this.gridX,
-                           y: this.gridY,
-                           scale_x: this.scale,
-                           scale_y: this.scale,
-                           time: Overview.ANIMATION_TIME,
-                           transition: 'easeOutQuad',
-                           onComplete: Lang.bind(this, this._fadeInAllOverlays)
-                         });
-    },
-
-    // Animates the addition of a new (empty) workspace
-    slideIn : function(oldScale) {
-        if (this.gridCol > this.gridRow) {
-            this.actor.set_position(global.screen_width, this.gridY);
-            this.actor.set_scale(oldScale, oldScale);
-        } else {
-            this.actor.set_position(this.gridX, global.screen_height);
-            this.actor.set_scale(this.scale, this.scale);
-        }
-        Tweener.addTween(this.actor,
-                         { x: this.gridX,
-                           y: this.gridY,
-                           scale_x: this.scale,
-                           scale_y: this.scale,
-                           time: Overview.ANIMATION_TIME,
-                           transition: 'easeOutQuad'
-                         });
-
-        this._visible = true;
-    },
-
-    // Animates the removal of a workspace
-    slideOut : function(onComplete) {
-        let destX = this.actor.x, destY = this.actor.y;
-
-        this._hideAllOverlays();
-
-        if (this.gridCol > this.gridRow)
-            destX = global.screen_width;
-        else
-            destY = global.screen_height;
-        Tweener.addTween(this.actor,
-                         { x: destX,
-                           y: destY,
-                           scale_x: this.scale,
-                           scale_y: this.scale,
-                           time: Overview.ANIMATION_TIME,
-                           transition: 'easeOutQuad',
-                           onComplete: onComplete
-                         });
-
-        this._visible = false;
-
-        // Don't let the user try to select this workspace as it's
-        // making its exit.
-        this._desktop.reactive = false;
-    },
-
     destroy : function() {
         this.actor.destroy();
     },
@@ -1486,7 +1278,7 @@ Workspace.prototype = {
         // their parent (this.actor), but we might have a zoomed window
         // which has been reparented to the stage - _windows[0] holds
         // the desktop window, which is never reparented
-        for (let w = 1; w < this._windows.length; w++)
+        for (let w = 0; w < this._windows.length; w++)
             this._windows[w].destroy();
         this._windows = [];
     },
@@ -1517,12 +1309,12 @@ Workspace.prototype = {
                       Lang.bind(this, this._onCloneSelected));
         clone.connect('drag-begin',
                       Lang.bind(this, function(clone) {
-                          this.emit('window-drag-begin', clone.actor);
+                          Main.overview.beginWindowDrag();
                           overlay.hide();
                       }));
         clone.connect('drag-end',
                       Lang.bind(this, function(clone) {
-                          this.emit('window-drag-end', clone.actor);
+                          Main.overview.endWindowDrag();
                           overlay.show();
                       }));
         clone.connect('zoom-start',
@@ -1545,7 +1337,7 @@ Workspace.prototype = {
     },
 
     _onShowOverlayClose: function (windowOverlay) {
-        for (let i = 1; i < this._windowOverlays.length; i++) {
+        for (let i = 0; i < this._windowOverlays.length; i++) {
             let overlay = this._windowOverlays[i];
             if (overlay == windowOverlay)
                 continue;
@@ -1589,6 +1381,15 @@ Workspace.prototype = {
     },
 
     // Draggable target interface
+    handleDragOver : function(source, actor, x, y, time) {
+        if (source instanceof WindowClone)
+            return DND.DragMotionResult.MOVE_DROP;
+        if (source.shellWorkspaceLaunch)
+            return DND.DragMotionResult.COPY_DROP;
+
+        return DND.DragMotionResult.CONTINUE;
+    },
+
     acceptDrop : function(source, actor, x, y, time) {
         if (source instanceof WindowClone) {
             let win = source.realWindow;
@@ -1645,11 +1446,11 @@ function _workspaceRelativeModifier(workspace) {
     }
 
     return [ { name: 'x',
-               parameters: { workspacePos: workspace.gridX,
+               parameters: { workspacePos: workspace.x,
                              overviewPos: overviewPosX,
                              overviewScale: overviewScale } },
              { name: 'y',
-               parameters: { workspacePos: workspace.gridY,
+               parameters: { workspacePos: workspace.y,
                              overviewPos: overviewPosY,
                              overviewScale: overviewScale } }
            ];
