@@ -7,6 +7,7 @@
 #include "shell-perf-log.h"
 #include "shell-window-tracker.h"
 #include "shell-wm.h"
+#include "st.h"
 
 #include "display.h"
 #include "util.h"
@@ -23,7 +24,8 @@
 #include <gio/gio.h>
 #include <math.h>
 #include <X11/extensions/Xfixes.h>
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <canberra.h>
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
@@ -33,7 +35,6 @@
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
 
 static void grab_notify (GtkWidget *widget, gboolean is_grab, gpointer user_data);
-static void update_root_window_pixmap (ShellGlobal *global);
 
 struct _ShellGlobal {
   GObject parent;
@@ -51,12 +52,13 @@ struct _ShellGlobal {
   XserverRegion input_region;
 
   GjsContext *js_context;
-  MutterPlugin *plugin;
+  MetaPlugin *plugin;
   ShellWM *wm;
   GSettings *settings;
   const char *datadir;
   const char *imagedir;
   const char *userdatadir;
+  StFocusManager *focus_manager;
 
   /* Displays the root window; see shell_global_create_root_pixmap_actor() */
   ClutterActor *root_pixmap;
@@ -67,6 +69,9 @@ struct _ShellGlobal {
   guint work_count;
   GSList *leisure_closures;
   guint leisure_function_id;
+
+  /* For sound notifications */
+  ca_context *sound_context;
 };
 
 enum {
@@ -80,11 +85,13 @@ enum {
   PROP_STAGE,
   PROP_STAGE_INPUT_MODE,
   PROP_WINDOW_GROUP,
+  PROP_BACKGROUND_ACTOR,
   PROP_WINDOW_MANAGER,
   PROP_SETTINGS,
   PROP_DATADIR,
   PROP_IMAGEDIR,
   PROP_USERDATADIR,
+  PROP_FOCUS_MANAGER,
 };
 
 G_DEFINE_TYPE(ShellGlobal, shell_global, G_TYPE_OBJECT);
@@ -120,7 +127,7 @@ shell_global_get_property(GObject         *object,
   switch (prop_id)
     {
     case PROP_OVERLAY_GROUP:
-      g_value_set_object (value, mutter_plugin_get_overlay_group (global->plugin));
+      g_value_set_object (value, meta_plugin_get_overlay_group (global->plugin));
       break;
     case PROP_SCREEN:
       g_value_set_object (value, shell_global_get_screen (global));
@@ -132,7 +139,7 @@ shell_global_get_property(GObject         *object,
       {
         int width, height;
 
-        mutter_plugin_query_screen_size (global->plugin, &width, &height);
+        meta_plugin_query_screen_size (global->plugin, &width, &height);
         g_value_set_int (value, width);
       }
       break;
@@ -140,18 +147,21 @@ shell_global_get_property(GObject         *object,
       {
         int width, height;
 
-        mutter_plugin_query_screen_size (global->plugin, &width, &height);
+        meta_plugin_query_screen_size (global->plugin, &width, &height);
         g_value_set_int (value, height);
       }
       break;
     case PROP_STAGE:
-      g_value_set_object (value, mutter_plugin_get_stage (global->plugin));
+      g_value_set_object (value, meta_plugin_get_stage (global->plugin));
       break;
     case PROP_STAGE_INPUT_MODE:
       g_value_set_enum (value, global->input_mode);
       break;
     case PROP_WINDOW_GROUP:
-      g_value_set_object (value, mutter_plugin_get_window_group (global->plugin));
+      g_value_set_object (value, meta_plugin_get_window_group (global->plugin));
+      break;
+    case PROP_BACKGROUND_ACTOR:
+      g_value_set_object (value, meta_plugin_get_background_actor (global->plugin));
       break;
     case PROP_WINDOW_MANAGER:
       g_value_set_object (value, global->wm);
@@ -168,6 +178,9 @@ shell_global_get_property(GObject         *object,
     case PROP_USERDATADIR:
       g_value_set_string (value, global->userdatadir);
       break;
+    case PROP_FOCUS_MANAGER:
+      g_value_set_object (value, global->focus_manager);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -214,6 +227,10 @@ shell_global_init (ShellGlobal *global)
 
   global->last_change_screen_width = 0;
   global->last_change_screen_height = 0;
+
+  ca_context_create (&global->sound_context);
+  ca_context_change_props (global->sound_context, CA_PROP_APPLICATION_NAME, PACKAGE_NAME, CA_PROP_APPLICATION_ID, "org.gnome.Shell", NULL);
+  ca_context_open (global->sound_context);
 }
 
 static void
@@ -284,6 +301,13 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         "Actor holding window actors",
                                                         CLUTTER_TYPE_ACTOR,
                                                         G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_BACKGROUND_ACTOR,
+                                   g_param_spec_object ("background-actor",
+                                                        "Background Actor",
+                                                        "Actor drawing root window background",
+                                                        CLUTTER_TYPE_ACTOR,
+                                                        G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_WINDOW_MANAGER,
                                    g_param_spec_object ("window-manager",
@@ -319,6 +343,13 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         "Directory containing gnome-shell user data",
                                                         NULL,
                                                         G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_FOCUS_MANAGER,
+                                   g_param_spec_object ("focus-manager",
+                                                        "Focus manager",
+                                                        "The shell's StFocusManager",
+                                                        ST_TYPE_FOCUS_MANAGER,
+                                                        G_PARAM_READABLE));
 }
 
 /**
@@ -383,9 +414,9 @@ shell_global_focus_stage (ShellGlobal *global)
   ClutterActor *stage;
   Window xstage;
 
-  stage = mutter_plugin_get_stage (global->plugin);
+  stage = meta_plugin_get_stage (global->plugin);
   xstage = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
-  xdpy = mutter_plugin_get_xdisplay (global->plugin);
+  xdpy = meta_plugin_get_xdisplay (global->plugin);
   XSetInputFocus (xdpy, xstage, RevertToPointerRoot,
                   shell_global_get_current_time (global));
 }
@@ -422,11 +453,11 @@ shell_global_set_stage_input_mode (ShellGlobal         *global,
   g_return_if_fail (SHELL_IS_GLOBAL (global));
 
   if (mode == SHELL_STAGE_INPUT_MODE_NONREACTIVE || global->gtk_grab_active)
-    mutter_plugin_set_stage_reactive (global->plugin, FALSE);
+    meta_plugin_set_stage_reactive (global->plugin, FALSE);
   else if (mode == SHELL_STAGE_INPUT_MODE_FULLSCREEN || !global->input_region)
-    mutter_plugin_set_stage_reactive (global->plugin, TRUE);
+    meta_plugin_set_stage_reactive (global->plugin, TRUE);
   else
-    mutter_plugin_set_stage_input_region (global->plugin, global->input_region);
+    meta_plugin_set_stage_input_region (global->plugin, global->input_region);
 
   if (mode == SHELL_STAGE_INPUT_MODE_FOCUSED)
     shell_global_focus_stage (global);
@@ -466,6 +497,9 @@ shell_global_set_cursor (ShellGlobal *global,
     case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
       name = "dnd-none";
       break;
+    case SHELL_CURSOR_POINTING_HAND:
+      name = "hand";
+      break;
     default:
       g_return_if_reached ();
     }
@@ -485,6 +519,8 @@ shell_global_set_cursor (ShellGlobal *global,
         case SHELL_CURSOR_DND_COPY:
           cursor_type = GDK_PLUS;
           break;
+        case SHELL_CURSOR_POINTING_HAND:
+          cursor_type = GDK_HAND2;
         case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
           cursor_type = GDK_X_CURSOR;
           break;
@@ -495,7 +531,7 @@ shell_global_set_cursor (ShellGlobal *global,
     }
   if (!global->stage_window)
     {
-      ClutterStage *stage = CLUTTER_STAGE (mutter_plugin_get_stage (global->plugin));
+      ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
 
       global->stage_window = gdk_window_foreign_new (clutter_x11_get_stage_window (stage));
     }
@@ -534,7 +570,7 @@ void
 shell_global_set_stage_input_region (ShellGlobal *global,
                                      GSList      *rectangles)
 {
-  MetaScreen *screen = mutter_plugin_get_screen (global->plugin);
+  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
   MetaDisplay *display = meta_screen_get_display (screen);
   Display *xdpy = meta_display_get_xdisplay (display);
   MetaRectangle *rect;
@@ -575,7 +611,7 @@ shell_global_set_stage_input_region (ShellGlobal *global,
 MetaScreen *
 shell_global_get_screen (ShellGlobal  *global)
 {
-  return mutter_plugin_get_screen (global->plugin);
+  return meta_plugin_get_screen (global->plugin);
 }
 
 /**
@@ -592,18 +628,18 @@ shell_global_get_gdk_screen (ShellGlobal *global)
 }
 
 /**
- * shell_global_get_windows:
+ * shell_global_get_window_actors:
  *
- * Gets the list of MutterWindows for the plugin's screen
+ * Gets the list of #MetaWindowActor for the plugin's screen
  *
- * Return value: (element-type Meta.MutterWindow) (transfer none): the list of windows
+ * Return value: (element-type Meta.WindowActor) (transfer none): the list of windows
  */
 GList *
-shell_global_get_windows (ShellGlobal *global)
+shell_global_get_window_actors (ShellGlobal *global)
 {
   g_return_val_if_fail (SHELL_IS_GLOBAL (global), NULL);
 
-  return mutter_plugin_get_windows (global->plugin);
+  return meta_plugin_get_window_actors (global->plugin);
 }
 
 static gboolean
@@ -612,7 +648,7 @@ update_screen_size (gpointer data)
   int width, height;
   ShellGlobal *global = SHELL_GLOBAL (data);
 
-  mutter_plugin_query_screen_size (global->plugin, &width, &height);
+  meta_plugin_query_screen_size (global->plugin, &width, &height);
 
   if (global->last_change_screen_width == width && global->last_change_screen_height == height)
     return FALSE;
@@ -675,8 +711,8 @@ global_stage_after_paint (ClutterStage *stage,
 }
 
 void
-_shell_global_set_plugin (ShellGlobal  *global,
-                          MutterPlugin *plugin)
+_shell_global_set_plugin (ShellGlobal *global,
+                          MetaPlugin  *plugin)
 {
   ClutterActor *stage;
   MetaScreen *screen;
@@ -688,7 +724,7 @@ _shell_global_set_plugin (ShellGlobal  *global,
   global->plugin = plugin;
   global->wm = shell_wm_new (plugin);
 
-  stage = mutter_plugin_get_stage (plugin);
+  stage = meta_plugin_get_stage (plugin);
 
   g_signal_connect (stage, "notify::width",
                     G_CALLBACK (global_stage_notify_width), global);
@@ -710,10 +746,12 @@ _shell_global_set_plugin (ShellGlobal  *global,
                                "End of stage page repaint",
                                "");
 
-  screen = mutter_plugin_get_screen (global->plugin);
+  screen = meta_plugin_get_screen (global->plugin);
   display = meta_screen_get_display (screen);
   g_signal_connect (display, "notify::focus-window",
                     G_CALLBACK (focus_window_changed), global);
+
+  global->focus_manager = st_focus_manager_get_for_stage (CLUTTER_STAGE (stage));
 }
 
 void
@@ -743,10 +781,10 @@ gboolean
 shell_global_begin_modal (ShellGlobal *global,
                           guint32      timestamp)
 {
-  ClutterStage *stage = CLUTTER_STAGE (mutter_plugin_get_stage (global->plugin));
+  ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
   Window stagewin = clutter_x11_get_stage_window (stage);
 
-  return mutter_plugin_begin_modal (global->plugin, stagewin, None, 0, timestamp);
+  return meta_plugin_begin_modal (global->plugin, stagewin, None, 0, timestamp);
 }
 
 /**
@@ -759,7 +797,7 @@ void
 shell_global_end_modal (ShellGlobal *global,
                         guint32      timestamp)
 {
-  mutter_plugin_end_modal (global->plugin, timestamp);
+  meta_plugin_end_modal (global->plugin, timestamp);
 }
 
 /**
@@ -780,7 +818,7 @@ shell_global_end_modal (ShellGlobal *global,
 gboolean
 shell_global_display_is_grabbed (ShellGlobal *global)
 {
-  MetaScreen *screen = mutter_plugin_get_screen (global->plugin);
+  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
   MetaDisplay *display = meta_screen_get_display (screen);
 
   return meta_display_get_grab_op (display) != META_GRAB_OP_NONE;
@@ -1099,77 +1137,6 @@ grab_notify (GtkWidget *widget, gboolean was_grabbed, gpointer user_data)
   shell_global_set_stage_input_mode (global, global->input_mode);
 }
 
-/*
- * Updates the global->root_pixmap actor with the root window's pixmap or fails
- * with a warning.
- */
-static void
-update_root_window_pixmap (ShellGlobal *global)
-{
-  Atom type;
-  int format;
-  gulong nitems;
-  gulong bytes_after;
-  guchar *data;
-  Pixmap root_pixmap_id = None;
-
-  if (!XGetWindowProperty (gdk_x11_get_default_xdisplay (),
-                           gdk_x11_get_default_root_xwindow (),
-                           gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID"),
-                           0, LONG_MAX,
-                           False,
-                           AnyPropertyType,
-                           &type, &format, &nitems, &bytes_after, &data) &&
-      type != None)
-  {
-     /* Got a property. */
-     if (type == XA_PIXMAP && format == 32 && nitems == 1)
-       {
-         /* Was what we expected. */
-         root_pixmap_id = *(Pixmap *)data;
-       }
-     else
-       {
-         g_warning ("Could not get the root window pixmap");
-       }
-
-     XFree(data);
-  }
-
-  clutter_x11_texture_pixmap_set_pixmap (CLUTTER_X11_TEXTURE_PIXMAP (global->root_pixmap),
-                                         root_pixmap_id);
-}
-
-/*
- * Called when the X server emits a root window change event. If the event is
- * about a new pixmap, update the global->root_pixmap actor.
- */
-static GdkFilterReturn
-root_window_filter (GdkXEvent *native, GdkEvent *event, gpointer data)
-{
-  XEvent *xevent = (XEvent *)native;
-
-  if ((xevent->type == PropertyNotify) &&
-      (xevent->xproperty.window == gdk_x11_get_default_root_xwindow ()) &&
-      (xevent->xproperty.atom == gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID")))
-    update_root_window_pixmap (SHELL_GLOBAL (data));
-
-  return GDK_FILTER_CONTINUE;
-}
-
-/*
- * Called when the root window pixmap actor is destroyed.
- */
-static void
-root_pixmap_destroy (GObject *sender, gpointer data)
-{
-  ShellGlobal *global = SHELL_GLOBAL (data);
-
-  gdk_window_remove_filter (gdk_get_default_root_window (),
-                            root_window_filter, global);
-  global->root_pixmap = NULL;
-}
-
 /**
  * shell_global_format_time_relative_pretty:
  * @global:
@@ -1218,80 +1185,6 @@ shell_global_format_time_relative_pretty (ShellGlobal *global,
    }
 }
 
-/**
- * shell_global_create_root_pixmap_actor:
- * @global: a #ShellGlobal
- *
- * Creates an actor showing the root window pixmap.
- *
- * Return value: (transfer none): a #ClutterActor with the root window pixmap.
- *               The actor is floating, hence (transfer none).
- */
-ClutterActor *
-shell_global_create_root_pixmap_actor (ShellGlobal *global)
-{
-  GdkWindow *window;
-  ClutterActor *stage;
-  ClutterColor stage_color;
-
-  /* The actor created is actually a ClutterClone of global->root_pixmap. */
-
-  if (global->root_pixmap == NULL)
-    {
-      global->root_pixmap = clutter_glx_texture_pixmap_new ();
-
-      clutter_actor_set_size (CLUTTER_ACTOR (global->root_pixmap),
-                              global->last_change_screen_width,
-                              global->last_change_screen_height);
-
-      clutter_texture_set_repeat (CLUTTER_TEXTURE (global->root_pixmap),
-                                  TRUE, TRUE);
-
-      /* The low and medium quality filters give nearest-neighbor resizing. */
-      clutter_texture_set_filter_quality (CLUTTER_TEXTURE (global->root_pixmap),
-                                          CLUTTER_TEXTURE_QUALITY_HIGH);
-
-      /* Initialize to the stage color, since that's what will be seen
-       * in the main view if there's no actual background window.
-       */
-      stage = mutter_plugin_get_stage (global->plugin);
-      clutter_stage_get_color (CLUTTER_STAGE (stage), &stage_color);
-      clutter_texture_set_from_rgb_data (CLUTTER_TEXTURE (global->root_pixmap),
-                                         /* ClutterColor has the same layout
-                                          * as one pixel of RGB(A) data.
-                                          */
-                                         (const guchar *)&stage_color, FALSE,
-                                         /* w, h, rowstride, bpp, flags */
-                                         1, 1, 3, 3, 0, NULL);
-
-      /* We can only clone an actor within a stage, so we hide the source
-       * texture then add it to the stage */
-      clutter_actor_hide (global->root_pixmap);
-      clutter_container_add_actor (CLUTTER_CONTAINER (stage),
-                                   global->root_pixmap);
-
-      /* This really should never happen; but just in case... */
-      g_signal_connect (global->root_pixmap, "destroy",
-                        G_CALLBACK (root_pixmap_destroy), global);
-
-      /* Metacity handles changes to some root window properties in its global
-       * event filter, though not _XROOTPMAP_ID. For all root window property
-       * changes, the global filter returns GDK_FILTER_CONTINUE, so our
-       * window specific filter will be called after the global one.
-       *
-       * Because Metacity is already handling root window property updates,
-       * we don't have to worry about adding the PropertyChange mask to the
-       * root window to get PropertyNotify events.
-       */
-      window = gdk_get_default_root_window ();
-      gdk_window_add_filter (window, root_window_filter, global);
-
-      update_root_window_pixmap (global);
-    }
-
-  return clutter_clone_new (global->root_pixmap);
-}
-
 /**
  * shell_global_get_monitors:
  * @global: the #ShellGlobal
@@ -1788,3 +1681,18 @@ shell_global_run_at_leisure (ShellGlobal         *global,
   if (global->work_count == 0)
     schedule_leisure_functions (global);
 }
+
+/**
+ * shell_global_play_theme_sound:
+ * @global: the #ShellGlobal
+ * @name: the sound name
+ *
+ * Plays a simple sound picked according to Freedesktop sound theme.
+ * Really just a workaround for libcanberra not being introspected.
+ */
+void
+shell_global_play_theme_sound (ShellGlobal *global,
+                               const char  *name)
+{
+  ca_context_play (global->sound_context, 0, CA_PROP_EVENT_ID, name, NULL);
+}
