@@ -136,6 +136,22 @@ st_align_get_type(void) {
     }
   return enum_type_id;
 }
+GType
+st_icon_type_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { ST_ICON_SYMBOLIC, "ST_ICON_SYMBOLIC", "symbolic" },
+        { ST_ICON_FULLCOLOR, "ST_ICON_FULLCOLOR", "fullcolor" },
+        { ST_ICON_APPLICATION, "ST_ICON_APPLICATION", "application" },
+        { ST_ICON_DOCUMENT, "ST_ICON_DOCUMENT", "document" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static (g_intern_static_string ("StIconType"), values);
+    }
+  return enum_type_id;
+}
 /* enumerations from "st/st-widget.h" */
 #include "st/st-widget.h"
 GType
