@@ -1,32 +1,30 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
-/* Drawing for StWidget.
-
-   Copyright (C) 2009,2010 Red Hat, Inc.
-
-   Contains code derived from:
-   rectangle.c: Rounded rectangle.
-   Copyright (C) 2008 litl, LLC.
-   st-shadow-texture.c: a class for creating soft shadow texture
-   Copyright (C) 2009 Florian Müllner <fmuellner@src.gnome.org>
-   st-texture-frame.h: Expandible texture actor
-   Copyright 2007 OpenedHand
-   Copyright 2009 Intel Corporation.
-
-   The St is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   The St is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with the St; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation, Inc., 59 Temple Place -
-   Suite 330, Boston, MA 02111-1307, USA.
-*/
+/*
+ * st-theme-node-drawing.c: Code to draw themed elements
+ *
+ * Copyright 2009, 2010 Red Hat, Inc.
+ * Copyright 2009, 2010 Florian Müllner
+ * Copyright 2010 Intel Corporation.
+ *
+ * Contains code derived from:
+ *   rectangle.c: Rounded rectangle.
+ *     Copyright 2008 litl, LLC.
+ *   st-texture-frame.h: Expandible texture actor
+ *     Copyright 2007 OpenedHand
+ *     Copyright 2009 Intel Corporation.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 2.1, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <stdlib.h>
 #include <math.h>
@@ -52,7 +50,7 @@ typedef struct {
 } StCornerSpec;
 
 static CoglHandle
-create_corner_texture (StCornerSpec *corner)
+create_corner_material (StCornerSpec *corner)
 {
   CoglHandle texture;
   cairo_t *cr;
@@ -167,7 +165,7 @@ load_corner (StTextureCache  *cache,
 {
   LoadCornerData *data = datap;
 
-  return create_corner_texture (data->corner);
+  return create_corner_material (data->corner);
 }
 
 /* To match the CSS specification, we want the border to look like it was
@@ -222,7 +220,7 @@ static CoglHandle
 st_theme_node_lookup_corner (StThemeNode    *node,
                              StCorner        corner_id)
 {
-  CoglHandle texture;
+  CoglHandle texture, material;
   char *key;
   StTextureCache *cache;
   StCornerSpec corner;
@@ -264,15 +262,22 @@ st_theme_node_lookup_corner (StThemeNode    *node,
         break;
     }
 
+  if (corner.color.alpha == 0 &&
+      corner.border_color_1.alpha == 0 &&
+      corner.border_color_2.alpha == 0)
+    return COGL_INVALID_HANDLE;
+
   key = corner_to_string (&corner);
 
   data.node = node;
   data.corner = &corner;
   texture = st_texture_cache_load (cache, key, ST_TEXTURE_CACHE_POLICY_NONE, load_corner, &data, NULL);
+  material = _st_create_texture_material (texture);
+  cogl_handle_unref (texture);
 
   g_free (key);
 
-  return texture;
+  return material;
 }
 
 static void
@@ -507,16 +512,20 @@ _st_theme_node_free_drawing_state (StThemeNode  *node)
 
   if (node->background_texture != COGL_INVALID_HANDLE)
     cogl_handle_unref (node->background_texture);
+  if (node->background_material != COGL_INVALID_HANDLE)
+    cogl_handle_unref (node->background_material);
   if (node->background_shadow_material != COGL_INVALID_HANDLE)
     cogl_handle_unref (node->background_shadow_material);
   if (node->border_texture != COGL_INVALID_HANDLE)
     cogl_handle_unref (node->border_texture);
+  if (node->border_material != COGL_INVALID_HANDLE)
+    cogl_handle_unref (node->border_material);
   if (node->border_shadow_material != COGL_INVALID_HANDLE)
     cogl_handle_unref (node->border_shadow_material);
 
   for (corner_id = 0; corner_id < 4; corner_id++)
-    if (node->corner_texture[corner_id] != COGL_INVALID_HANDLE)
-      cogl_handle_unref (node->corner_texture[corner_id]);
+    if (node->corner_material[corner_id] != COGL_INVALID_HANDLE)
+      cogl_handle_unref (node->corner_material[corner_id]);
 
   _st_theme_node_init_drawing_state (node);
 }
@@ -527,12 +536,14 @@ _st_theme_node_init_drawing_state (StThemeNode *node)
   int corner_id;
 
   node->background_texture = COGL_INVALID_HANDLE;
+  node->background_material = COGL_INVALID_HANDLE;
   node->background_shadow_material = COGL_INVALID_HANDLE;
   node->border_shadow_material = COGL_INVALID_HANDLE;
   node->border_texture = COGL_INVALID_HANDLE;
+  node->border_material = COGL_INVALID_HANDLE;
 
   for (corner_id = 0; corner_id < 4; corner_id++)
-    node->corner_texture[corner_id] = COGL_INVALID_HANDLE;
+    node->corner_material[corner_id] = COGL_INVALID_HANDLE;
 }
 
 static void st_theme_node_paint_borders (StThemeNode           *node,
@@ -581,6 +592,11 @@ st_theme_node_render_resources (StThemeNode   *node,
       node->border_texture = st_theme_node_render_gradient (node);
     }
 
+  if (node->border_texture)
+    node->border_material = _st_create_texture_material (node->border_texture);
+  else
+    node->border_material = COGL_INVALID_HANDLE;
+
   if (shadow_spec)
     {
       if (node->border_texture != COGL_INVALID_HANDLE)
@@ -622,6 +638,7 @@ st_theme_node_render_resources (StThemeNode   *node,
     {
 
       node->background_texture = st_texture_cache_load_file_to_cogl_texture (texture_cache, background_image);
+      node->background_material = _st_create_texture_material (node->background_texture);
 
       if (shadow_spec)
         {
@@ -630,40 +647,26 @@ st_theme_node_render_resources (StThemeNode   *node,
         }
     }
 
-  node->corner_texture[ST_CORNER_TOPLEFT] =
+  node->corner_material[ST_CORNER_TOPLEFT] =
     st_theme_node_lookup_corner (node, ST_CORNER_TOPLEFT);
-  node->corner_texture[ST_CORNER_TOPRIGHT] =
+  node->corner_material[ST_CORNER_TOPRIGHT] =
     st_theme_node_lookup_corner (node, ST_CORNER_TOPRIGHT);
-  node->corner_texture[ST_CORNER_BOTTOMRIGHT] =
+  node->corner_material[ST_CORNER_BOTTOMRIGHT] =
     st_theme_node_lookup_corner (node, ST_CORNER_BOTTOMRIGHT);
-  node->corner_texture[ST_CORNER_BOTTOMLEFT] =
+  node->corner_material[ST_CORNER_BOTTOMLEFT] =
     st_theme_node_lookup_corner (node, ST_CORNER_BOTTOMLEFT);
 }
 
 static void
-paint_texture_with_opacity (CoglHandle       texture,
-                            ClutterActorBox *box,
-                            guint8           paint_opacity)
+paint_material_with_opacity (CoglHandle       material,
+                             ClutterActorBox *box,
+                             guint8           paint_opacity)
 {
-  if (paint_opacity == 255)
-    {
-      /* Minor: optimization use the default material if we can */
-      cogl_set_source_texture (texture);
-      cogl_rectangle (box->x1, box->y1, box->x2, box->y2);
-      return;
-    }
-
-  CoglHandle material;
-
-  material = cogl_material_new ();
-  cogl_material_set_layer (material, 0, texture);
   cogl_material_set_color4ub (material,
                               paint_opacity, paint_opacity, paint_opacity, paint_opacity);
 
   cogl_set_source (material);
   cogl_rectangle (box->x1, box->y1, box->x2, box->y2);
-
-  cogl_handle_unref (material);
 }
 
 static void
@@ -678,7 +681,7 @@ st_theme_node_paint_borders (StThemeNode           *node,
   int max_width_radius[4];
   int corner_id;
   ClutterColor border_color;
-  CoglHandle material;
+  guint8 alpha;
 
   width = box->x2 - box->x1;
   height = box->y2 - box->y1;
@@ -700,72 +703,72 @@ st_theme_node_paint_borders (StThemeNode           *node,
       float x1, y1, x2, y2;
 
       over (&border_color, &node->background_color, &effective_border);
+      alpha = paint_opacity * effective_border.alpha / 255;
 
-      cogl_set_source_color4ub (effective_border.red,
-                                effective_border.green,
-                                effective_border.blue,
-                                paint_opacity * effective_border.alpha / 255);
+      if (alpha > 0)
+        {
+          cogl_set_source_color4ub (effective_border.red,
+                                    effective_border.green,
+                                    effective_border.blue,
+                                    alpha);
 
-      /* NORTH */
-      skip_corner_1 = node->border_radius[ST_CORNER_TOPLEFT] > 0;
-      skip_corner_2 = node->border_radius[ST_CORNER_TOPRIGHT] > 0;
+          /* NORTH */
+          skip_corner_1 = node->border_radius[ST_CORNER_TOPLEFT] > 0;
+          skip_corner_2 = node->border_radius[ST_CORNER_TOPRIGHT] > 0;
 
-      x1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPLEFT] : 0;
-      y1 = 0;
-      x2 = skip_corner_2 ? width - max_width_radius[ST_CORNER_TOPRIGHT] : width;
-      y2 = border_width;
-      cogl_rectangle (x1, y1, x2, y2);
+          x1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPLEFT] : 0;
+          y1 = 0;
+          x2 = skip_corner_2 ? width - max_width_radius[ST_CORNER_TOPRIGHT] : width;
+          y2 = border_width;
+          cogl_rectangle (x1, y1, x2, y2);
 
-      /* EAST */
-      skip_corner_1 = node->border_radius[ST_CORNER_TOPRIGHT] > 0;
-      skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
+          /* EAST */
+          skip_corner_1 = node->border_radius[ST_CORNER_TOPRIGHT] > 0;
+          skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
 
-      x1 = width - border_width;
-      y1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPRIGHT] : border_width;
-      x2 = width;
-      y2 = skip_corner_2 ? height - max_width_radius[ST_CORNER_BOTTOMRIGHT]
-                         : height - border_width;
-      cogl_rectangle (x1, y1, x2, y2);
+          x1 = width - border_width;
+          y1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPRIGHT] : border_width;
+          x2 = width;
+          y2 = skip_corner_2 ? height - max_width_radius[ST_CORNER_BOTTOMRIGHT]
+                             : height - border_width;
+          cogl_rectangle (x1, y1, x2, y2);
 
-      /* SOUTH */
-      skip_corner_1 = node->border_radius[ST_CORNER_BOTTOMLEFT] > 0;
-      skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
+          /* SOUTH */
+          skip_corner_1 = node->border_radius[ST_CORNER_BOTTOMLEFT] > 0;
+          skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
 
-      x1 = skip_corner_1 ? max_width_radius[ST_CORNER_BOTTOMLEFT] : 0;
-      y1 = height - border_width;
-      x2 = skip_corner_2 ? width - max_width_radius[ST_CORNER_BOTTOMRIGHT]
-                         : width;
-      y2 = height;
-      cogl_rectangle (x1, y1, x2, y2);
+          x1 = skip_corner_1 ? max_width_radius[ST_CORNER_BOTTOMLEFT] : 0;
+          y1 = height - border_width;
+          x2 = skip_corner_2 ? width - max_width_radius[ST_CORNER_BOTTOMRIGHT]
+                             : width;
+          y2 = height;
+          cogl_rectangle (x1, y1, x2, y2);
 
-      /* WEST */
-      skip_corner_1 = node->border_radius[ST_CORNER_TOPLEFT] > 0;
-      skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMLEFT] > 0;
+          /* WEST */
+          skip_corner_1 = node->border_radius[ST_CORNER_TOPLEFT] > 0;
+          skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMLEFT] > 0;
 
-      x1 = 0;
-      y1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPLEFT] : border_width;
-      x2 = border_width;
-      y2 = skip_corner_2 ? height - max_width_radius[ST_CORNER_BOTTOMLEFT]
-                         : height - border_width;
-      cogl_rectangle (x1, y1, x2, y2);
+          x1 = 0;
+          y1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPLEFT] : border_width;
+          x2 = border_width;
+          y2 = skip_corner_2 ? height - max_width_radius[ST_CORNER_BOTTOMLEFT]
+                             : height - border_width;
+          cogl_rectangle (x1, y1, x2, y2);
+        }
     }
 
   /* corners */
-  if (max_border_radius > 0)
+  if (max_border_radius > 0 && paint_opacity > 0)
     {
-      material = cogl_material_new ();
-      cogl_material_set_color4ub (material,
-                                  paint_opacity, paint_opacity,
-                                  paint_opacity, paint_opacity);
-      cogl_set_source (material);
-
       for (corner_id = 0; corner_id < 4; corner_id++)
         {
-          if (node->corner_texture[corner_id] == COGL_INVALID_HANDLE)
+          if (node->corner_material[corner_id] == COGL_INVALID_HANDLE)
             continue;
 
-          cogl_material_set_layer (material,
-                                   0, node->corner_texture[corner_id]);
+          cogl_material_set_color4ub (node->corner_material[corner_id],
+                                      paint_opacity, paint_opacity,
+                                      paint_opacity, paint_opacity);
+          cogl_set_source (node->corner_material[corner_id]);
 
           switch (corner_id)
             {
@@ -791,109 +794,112 @@ st_theme_node_paint_borders (StThemeNode           *node,
                 break;
             }
         }
-      cogl_handle_unref (material);
     }
 
   /* background color */
-  cogl_set_source_color4ub (node->background_color.red,
-                            node->background_color.green,
-                            node->background_color.blue,
-                            paint_opacity * node->background_color.alpha / 255);
-
-  /* We add padding to each corner, so that all corners end up as if they
-   * had a border-radius of max_border_radius, which allows us to treat
-   * corners as uniform further on.
-   */
-  for (corner_id = 0; corner_id < 4; corner_id++)
+  alpha = paint_opacity * node->background_color.alpha / 255;
+  if (alpha > 0)
     {
-      float verts[8];
-      int n_rects;
+      cogl_set_source_color4ub (node->background_color.red,
+                                node->background_color.green,
+                                node->background_color.blue,
+                                alpha);
 
-      /* corner texture does not need padding */
-      if (max_border_radius == node->border_radius[corner_id])
-        continue;
-
-      n_rects = node->border_radius[corner_id] == 0 ? 1 : 2;
-
-      switch (corner_id)
-        {
-          case ST_CORNER_TOPLEFT:
-            verts[0] = border_width;
-            verts[1] = max_width_radius[ST_CORNER_TOPLEFT];
-            verts[2] = max_border_radius;
-            verts[3] = max_border_radius;
-            if (n_rects == 2)
-              {
-                verts[4] = max_width_radius[ST_CORNER_TOPLEFT];
-                verts[5] = border_width;
-                verts[6] = max_border_radius;
-                verts[7] = max_width_radius[ST_CORNER_TOPLEFT];
-              }
-            break;
-          case ST_CORNER_TOPRIGHT:
-            verts[0] = width - max_border_radius;
-            verts[1] = max_width_radius[ST_CORNER_TOPRIGHT];
-            verts[2] = width - border_width;
-            verts[3] = max_border_radius;
-            if (n_rects == 2)
-              {
-                verts[4] = width - max_border_radius;
-                verts[5] = border_width;
-                verts[6] = width - max_width_radius[ST_CORNER_TOPRIGHT];
-                verts[7] = max_width_radius[ST_CORNER_TOPRIGHT];
-              }
-            break;
-          case ST_CORNER_BOTTOMRIGHT:
-            verts[0] = width - max_border_radius;
-            verts[1] = height - max_border_radius;
-            verts[2] = width - border_width;
-            verts[3] = height - max_width_radius[ST_CORNER_BOTTOMRIGHT];
-            if (n_rects == 2)
-              {
-                verts[4] = width - max_border_radius;
-                verts[5] = height - max_width_radius[ST_CORNER_BOTTOMRIGHT];
-                verts[6] = width - max_width_radius[ST_CORNER_BOTTOMRIGHT];
-                verts[7] = height - border_width;
-              }
-            break;
-          case ST_CORNER_BOTTOMLEFT:
-            verts[0] = border_width;
-            verts[1] = height - max_border_radius;
-            verts[2] = max_border_radius;
-            verts[3] = height - max_width_radius[ST_CORNER_BOTTOMLEFT];
-            if (n_rects == 2)
-              {
-                verts[4] = max_width_radius[ST_CORNER_BOTTOMLEFT];
-                verts[5] = height - max_width_radius[ST_CORNER_BOTTOMLEFT];
-                verts[6] = max_border_radius;
-                verts[7] = height - border_width;
-              }
-            break;
-        }
-      cogl_rectangles (verts, n_rects);
-    }
-
-  if (max_border_radius > border_width)
-    {
-      /* Once we've drawn the borders and corners, if the corners are bigger
-       * the the border width, the remaining area is shaped like
-       *
-       *  ########
-       * ##########
-       * ##########
-       *  ########
-       *
-       * We draw it in 3 pieces - first the top and bottom, then the main
-       * rectangle
+      /* We add padding to each corner, so that all corners end up as if they
+       * had a border-radius of max_border_radius, which allows us to treat
+       * corners as uniform further on.
        */
-      cogl_rectangle (max_border_radius, border_width,
-                      width - max_border_radius, max_border_radius);
-      cogl_rectangle (max_border_radius, height - max_border_radius,
-                      width - max_border_radius, height - border_width);
-    }
+      for (corner_id = 0; corner_id < 4; corner_id++)
+        {
+          float verts[8];
+          int n_rects;
 
-  cogl_rectangle (border_width, MAX(border_width, max_border_radius),
-                  width - border_width, height - MAX(border_width, max_border_radius));
+          /* corner texture does not need padding */
+          if (max_border_radius == node->border_radius[corner_id])
+            continue;
+
+          n_rects = node->border_radius[corner_id] == 0 ? 1 : 2;
+
+          switch (corner_id)
+            {
+              case ST_CORNER_TOPLEFT:
+                verts[0] = border_width;
+                verts[1] = max_width_radius[ST_CORNER_TOPLEFT];
+                verts[2] = max_border_radius;
+                verts[3] = max_border_radius;
+                if (n_rects == 2)
+                  {
+                    verts[4] = max_width_radius[ST_CORNER_TOPLEFT];
+                    verts[5] = border_width;
+                    verts[6] = max_border_radius;
+                    verts[7] = max_width_radius[ST_CORNER_TOPLEFT];
+                  }
+                break;
+              case ST_CORNER_TOPRIGHT:
+                verts[0] = width - max_border_radius;
+                verts[1] = max_width_radius[ST_CORNER_TOPRIGHT];
+                verts[2] = width - border_width;
+                verts[3] = max_border_radius;
+                if (n_rects == 2)
+                  {
+                    verts[4] = width - max_border_radius;
+                    verts[5] = border_width;
+                    verts[6] = width - max_width_radius[ST_CORNER_TOPRIGHT];
+                    verts[7] = max_width_radius[ST_CORNER_TOPRIGHT];
+                  }
+                break;
+              case ST_CORNER_BOTTOMRIGHT:
+                verts[0] = width - max_border_radius;
+                verts[1] = height - max_border_radius;
+                verts[2] = width - border_width;
+                verts[3] = height - max_width_radius[ST_CORNER_BOTTOMRIGHT];
+                if (n_rects == 2)
+                  {
+                    verts[4] = width - max_border_radius;
+                    verts[5] = height - max_width_radius[ST_CORNER_BOTTOMRIGHT];
+                    verts[6] = width - max_width_radius[ST_CORNER_BOTTOMRIGHT];
+                    verts[7] = height - border_width;
+                  }
+                break;
+              case ST_CORNER_BOTTOMLEFT:
+                verts[0] = border_width;
+                verts[1] = height - max_border_radius;
+                verts[2] = max_border_radius;
+                verts[3] = height - max_width_radius[ST_CORNER_BOTTOMLEFT];
+                if (n_rects == 2)
+                  {
+                    verts[4] = max_width_radius[ST_CORNER_BOTTOMLEFT];
+                    verts[5] = height - max_width_radius[ST_CORNER_BOTTOMLEFT];
+                    verts[6] = max_border_radius;
+                    verts[7] = height - border_width;
+                  }
+                break;
+            }
+          cogl_rectangles (verts, n_rects);
+        }
+
+      if (max_border_radius > border_width)
+        {
+          /* Once we've drawn the borders and corners, if the corners are bigger
+           * the the border width, the remaining area is shaped like
+           *
+           *  ########
+           * ##########
+           * ##########
+           *  ########
+           *
+           * We draw it in 3 pieces - first the top and bottom, then the main
+           * rectangle
+           */
+          cogl_rectangle (max_border_radius, border_width,
+                          width - max_border_radius, max_border_radius);
+          cogl_rectangle (max_border_radius, height - max_border_radius,
+                          width - max_border_radius, height - border_width);
+        }
+
+      cogl_rectangle (border_width, MAX(border_width, max_border_radius),
+                      width - border_width, height - MAX(border_width, max_border_radius));
+    }
 }
 
 static void
@@ -930,8 +936,7 @@ st_theme_node_paint_sliced_border_image (StThemeNode           *node,
   if (ey < 0)
     ey = border_bottom;          /* FIXME ? */
 
-  material = cogl_material_new ();
-  cogl_material_set_layer (material, 0, node->border_texture);
+  material = node->border_material;
   cogl_material_set_color4ub (material,
                               paint_opacity, paint_opacity, paint_opacity, paint_opacity);
 
@@ -988,8 +993,6 @@ st_theme_node_paint_sliced_border_image (StThemeNode           *node,
 
     cogl_rectangles_with_texture_coords (rectangles, 9);
   }
-
-  cogl_handle_unref (material);
 }
 
 static void
@@ -1096,11 +1099,11 @@ st_theme_node_paint (StThemeNode           *node,
                                    &allocation,
                                    paint_opacity);
 
-  if (node->border_texture != COGL_INVALID_HANDLE)
+  if (node->border_material != COGL_INVALID_HANDLE)
     {
       /* Gradients and border images are mutually exclusive at this time */
       if (node->background_gradient_type != ST_GRADIENT_NONE)
-        paint_texture_with_opacity (node->border_texture, &allocation, paint_opacity);
+        paint_material_with_opacity (node->border_material, &allocation, paint_opacity);
       else
         st_theme_node_paint_sliced_border_image (node, &allocation, paint_opacity);
     }
@@ -1133,7 +1136,7 @@ st_theme_node_paint (StThemeNode           *node,
                                        &background_box,
                                        paint_opacity);
 
-      paint_texture_with_opacity (node->background_texture, &background_box, paint_opacity);
+      paint_material_with_opacity (node->background_material, &background_box, paint_opacity);
     }
 }
 
@@ -1170,9 +1173,13 @@ st_theme_node_copy_cached_paint_state (StThemeNode *node,
     node->border_shadow_material = cogl_handle_ref (other->border_shadow_material);
   if (other->background_texture)
     node->background_texture = cogl_handle_ref (other->background_texture);
+  if (other->background_material)
+    node->background_material = cogl_handle_ref (other->background_material);
   if (other->border_texture)
     node->border_texture = cogl_handle_ref (other->border_texture);
+  if (other->border_material)
+    node->border_material = cogl_handle_ref (other->border_material);
   for (corner_id = 0; corner_id < 4; corner_id++)
-    if (other->corner_texture[corner_id])
-      node->corner_texture[corner_id] = cogl_handle_ref (other->corner_texture[corner_id]);
+    if (other->corner_material[corner_id])
+      node->corner_material[corner_id] = cogl_handle_ref (other->corner_material[corner_id]);
 }
